/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/sys by iffe version 2024-03-20 : : */
#ifndef _AST_SYS_H
#define _AST_SYS_H	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#include <stdlib.h>
#include <stddef.h>
#include <sys/types.h>
#include <stdint.h>
#include <inttypes.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <fcntl.h>
#include <locale.h>
#define _hdr_time	1	/* #include <time.h> ok */
#define _sys_time	1	/* #include <sys/time.h> ok */
#define _sys_times	1	/* #include <sys/times.h> ok */
#define _typ_dev_t	1	/* dev_t is a type */
#define _typ_nlink_t	1	/* nlink_t is a type */
#define _typ_gid_t	1	/* gid_t is a type */
#define _typ_mode_t	1	/* mode_t is a type */
#define _typ_uid_t	1	/* uid_t is a type */
#define _hdr_stdio	1	/* #include <stdio.h> ok */
#define _hdr_wchar	1	/* #include <wchar.h> ok */
#define _typ_wchar_t	1	/* wchar_t is a type */
#define _typ_pid_t	1	/* pid_t is a type */
#define _typ_ssize_t	1	/* ssize_t is a type */
#define _typ_wint_t	1	/* wint_t is a type */
#define _sys_socket	1	/* #include <sys/socket.h> ok */
#define _typ_socklen_t	1	/* socklen_t is a type */
#define _typ_size_t	1	/* size_t is a type */
#define _typ_clock_t	1	/* clock_t is a type */
#define _typ_ino_t	1	/* ino_t is a type */
#define _typ_off_t	1	/* off_t is a type */
#define _typ_ptrdiff_t	1	/* ptrdiff_t is a type */
#define _typ_time_t	1	/* time_t is a type */
#define _typ_div_t	1	/* div_t is a type */
#define _typ_ldiv_t	1	/* ldiv_t is a type */
extern void	cfree(void*);
extern int	eaccess(const char*, int);
extern char*	gettxt(const char*, const char*);
extern void*	memalign(size_t, size_t);
extern void*	pvalloc(size_t);
extern char*	resolvepath(const char*, char*, size_t);
extern pid_t spawnveg(const char*, char* const[], char* const[], pid_t, int);
#undef extern
#include <stdarg.h>
#endif
