/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/dirent by iffe version 2024-03-20 : : */
#ifndef _def_dirent_ast
#define _def_dirent_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#define _lib_opendir	1	/* opendir() in default lib(s) */
#define _hdr_dirent	1	/* #include <dirent.h> ok */
#define _sys_dir	1	/* #include <sys/dir.h> ok */
#define _nxt_dirent <../include/dirent.h>	/* include path for the native <dirent.h> */
#define _nxt_dirent_str "../include/dirent.h"	/* include string for the native <dirent.h> */
/*
 * <dirent.h> for systems with ok <dirent.h>
 */

#ifndef _DIRENT_H

#include <../include/dirent.h>	/* the native <dirent.h> */

#ifndef _DIRENT_H
#define _DIRENT_H
#endif

#endif
#endif
