/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/sizeof by iffe version 2024-03-20 : : */
#ifndef _def_sizeof_ast
#define _def_sizeof_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#define _ast_sizeof_char	1
#define _ast_sizeof_short	2
#define _ast_sizeof_int		4
#define _ast_sizeof_long	4
#define _ast_sizeof_intmax_t	8
#endif
