/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/ndbm by iffe version 2024-03-20 : : */
#ifndef _def_ndbm_ast
#define _def_ndbm_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#define _hdr_ndbm	1	/* #include <ndbm.h> ok */
#include <ndbm.h>
#define _use_ndbm	1

#endif
