/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/map.c by iffe version 2024-03-20 : : */
#ifndef _def_map_ast
#define _def_map_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
/*
 * prototypes provided for standard interfaces hijacked
 * by AST and mapped to _ast_* but already prototyped
 * unmapped in native headers included by <ast_std.h>
 */

#undef	glob
#define glob		_ast_glob
#undef	globfree
#define globfree	_ast_globfree
#undef	signal
#define signal      	_ast_signal
#undef	regaddclass
#define regaddclass	_ast_regaddclass
#undef	regalloc
#define regalloc	_ast_regalloc
#undef	regcache
#define regcache	_ast_regcache
#undef	regclass
#define regclass	_ast_regclass
#undef	regcmp
#define regcmp		_ast_regcmp
#undef	regcollate
#define regcollate      _ast_regcollate
#undef	regcomb
#define regcomb		_ast_regcomb
#undef	regcomp
#define regcomp		_ast_regcomp
#undef	regdecomp
#define regdecomp	_ast_regdecomp
#undef	regdup
#define regdup		_ast_regdup
#undef	regerror
#define regerror	_ast_regerror
#undef	regex
#define regex		_ast_regex
#undef	regexec
#define regexec		_ast_regexec
#undef	regfatal
#define regfatal	_ast_regfatal
#undef	regfatalpat
#define regfatalpat     _ast_regfatalpat
#undef	regfree
#define regfree		_ast_regfree
#undef	regncomp
#define regncomp	_ast_regncomp
#undef	regnexec
#define regnexec	_ast_regnexec
#undef	regrecord
#define regrecord       _ast_regrecord
#undef	regrexec
#define regrexec	_ast_regrexec
#undef	regstat
#define regstat		_ast_regstat
#undef	regsub
#define regsub		_ast_regsub
#undef	regsubcomp
#define regsubcomp	_ast_regsubcomp
#undef	regsubexec
#define regsubexec	_ast_regsubexec
#undef	regsubflags
#define regsubflags	_ast_regsubflags
#undef	regsubfree
#define regsubfree	_ast_regsubfree
#undef	strmode
#define strmode		_ast_strmode
#undef	strdup
#define strdup		_ast_strdup
extern char*		strdup(const char*);
#undef	extern
#undef	extern

#undef	extern
#endif
