/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/lib by iffe version 2024-03-20 : : */
#ifndef _def_lib_ast
#define _def_lib_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#define _sys_mman	1	/* #include <sys/mman.h> ok */
#define _hdr_fcntl	1	/* #include <fcntl.h> ok */
#define _hdr_dirent	1	/* #include <dirent.h> ok */
#define _hdr_fnmatch	1	/* #include <fnmatch.h> ok */
#define _hdr_libgen	1	/* #include <libgen.h> ok */
#define _hdr_limits	1	/* #include <limits.h> ok */
#define _hdr_locale	1	/* #include <locale.h> ok */
#define _hdr_nl_types	1	/* #include <nl_types.h> ok */
#define _hdr_spawn	1	/* #include <spawn.h> ok */
#define _hdr_utime	1	/* #include <utime.h> ok */
#define _hdr_wctype	1	/* #include <wctype.h> ok */
#define _hdr_wchar	1	/* <wchar.h> and isw*() really work */
#define _dat_tzname	1	/* tzname in default lib(s) */
#define _lib_bcopy	1	/* bcopy() in default lib(s) */
#define _lib_bzero	1	/* bzero() in default lib(s) */
#define _lib_confstr	1	/* confstr() in default lib(s) */
#define _lib_fchmod	1	/* fchmod() in default lib(s) */
#define _lib_fcntl	1	/* fcntl() in default lib(s) */
#define _lib_fnmatch	1	/* fnmatch() in default lib(s) */
#define _lib_fork	1	/* fork() in default lib(s) */
#define _lib_fsync	1	/* fsync() in default lib(s) */
#define _lib_getdents	1	/* getdents() in default lib(s) */
#define _lib_getdtablesize	1	/* getdtablesize() in default lib(s) */
#define _lib_gethostname	1	/* gethostname() in default lib(s) */
#define _lib_getpagesize	1	/* getpagesize() in default lib(s) */
#define _lib_getrlimit	1	/* getrlimit() in default lib(s) */
#define _lib_glob	1	/* glob() in default lib(s) */
#define _lib_iswblank	1	/* iswblank() in default lib(s) */
#define _lib_iswctype	1	/* iswctype() in default lib(s) */
#define _lib_killpg	1	/* killpg() in default lib(s) */
#define _lib_link	1	/* link() in default lib(s) */
#define _lib_localeconv	1	/* localeconv() in default lib(s) */
#define _lib_madvise	1	/* madvise() in default lib(s) */
#define _lib_mbtowc	1	/* mbtowc() in default lib(s) */
#define _lib_mbrtowc	1	/* mbrtowc() in default lib(s) */
#define _lib_mkdir	1	/* mkdir() in default lib(s) */
#define _lib_mkfifo	1	/* mkfifo() in default lib(s) */
#define _lib_mktemp	1	/* mktemp() in default lib(s) */
#define _lib_mktime	1	/* mktime() in default lib(s) */
#define _lib_mount	1	/* mount() in default lib(s) */
#define _lib_opendir	1	/* opendir() in default lib(s) */
#define _lib_pathconf	1	/* pathconf() in default lib(s) */
#define _lib_readlink	1	/* readlink() in default lib(s) */
#define _lib_remove	1	/* remove() in default lib(s) */
#define _lib_rename	1	/* rename() in default lib(s) */
#define _lib_rewinddir	1	/* rewinddir() in default lib(s) */
#define _lib_rmdir	1	/* rmdir() in default lib(s) */
#define _lib_setlocale	1	/* setlocale() in default lib(s) */
#define _lib_setpgrp	1	/* setpgrp() in default lib(s) */
#define _lib_setreuid	1	/* setreuid() in default lib(s) */
#define _lib_setuid	1	/* setuid() in default lib(s) */
#define _lib_sigaction	1	/* sigaction() in default lib(s) */
#define _lib_sigprocmask	1	/* sigprocmask() in default lib(s) */
#define _lib_sigsetmask	1	/* sigsetmask() in default lib(s) */
#define _lib_sigvec	1	/* sigvec() in default lib(s) */
#define _lib_socketpair	1	/* socketpair() in default lib(s) */
#define _lib_strcoll	1	/* strcoll() in default lib(s) */
#define _lib_strdup	1	/* strdup() in default lib(s) */
#define _lib_strerror	1	/* strerror() in default lib(s) */
#define _lib_strcasecmp	1	/* strcasecmp() in default lib(s) */
#define _lib_strncasecmp	1	/* strncasecmp() in default lib(s) */
#define _lib_strlcat	1	/* strlcat() in default lib(s) */
#define _lib_strlcpy	1	/* strlcpy() in default lib(s) */
#define _lib_strmode	1	/* strmode() in default lib(s) */
#define _lib_strxfrm	1	/* strxfrm() in default lib(s) */
#define _lib_strftime	1	/* strftime() in default lib(s) */
#define _lib_swab	1	/* swab() in default lib(s) */
#define _lib_symlink	1	/* symlink() in default lib(s) */
#define _lib_sysconf	1	/* sysconf() in default lib(s) */
#define _lib_telldir	1	/* telldir() in default lib(s) */
#define _lib_tmpnam	1	/* tmpnam() in default lib(s) */
#define _lib_tzset	1	/* tzset() in default lib(s) */
#define _lib_unlink	1	/* unlink() in default lib(s) */
#define _lib_utime	1	/* utime() in default lib(s) */
#define _lib_wctype	1	/* wctype() in default lib(s) */
#define _lib_ftruncate	1	/* ftruncate() in default lib(s) */
#define _lib_truncate	1	/* truncate() in default lib(s) */
#define _hdr_stdlib	1	/* #include <stdlib.h> ok */
#define _lib_strtod	1	/* strtod() in default lib(s) */
#define _lib_strtold	1	/* strtold() in default lib(s) */
#define _lib_strtol	1	/* strtol() in default lib(s) */
#define _lib_strtoll	1	/* strtoll() in default lib(s) */
#define _lib_strtoul	1	/* strtoul() in default lib(s) */
#define _lib_strtoull	1	/* strtoull() in default lib(s) */
#define _hdr_signal	1	/* #include <signal.h> ok */
#define _npt_sigflag	1	/* sigflag() needs a prototype */
#define _sys_dir	1	/* #include <sys/dir.h> ok */
#define _mem_d_reclen_direct	1	/* d_reclen is a member of struct direct */
#define _mem_d_fileno_dirent	1	/* d_fileno is a member of struct dirent */
#define _mem_d_ino_dirent	1	/* d_ino is a member of struct dirent */
#define _mem_d_namlen_dirent	1	/* d_namlen is a member of struct dirent */
#define _mem_d_off_dirent	1	/* d_off is a member of struct dirent */
#define _mem_d_reclen_dirent	1	/* d_reclen is a member of struct dirent */
#define _mem_d_type_dirent	1	/* d_type is a member of struct dirent */
#define _hdr_time	1	/* #include <time.h> ok */
#define _mem_tm_zone_tm	1	/* tm_zone is a member of struct tm */
#define _sys_filio	1	/* #include <sys/filio.h> ok */
#define _sys_ioctl	1	/* #include <sys/ioctl.h> ok */
#define _sys_resource	1	/* #include <sys/resource.h> ok */
#define _sys_socket	1	/* #include <sys/socket.h> ok */
#define _tst_errno	1	/* errno can be assigned */
#define _lib_poll	1	/* poll() args comply with the POSIX standard */
#define _sys_select	1	/* #include <sys/select.h> ok */
#define _lib_select	1	/* select() has standard 5 arg interface */
#define _sys_select	1	/* select() requires <sys/select.h> */
#define _pipe_rw	1	/* full duplex pipes */
#define _hdr_unistd	1	/* #include <unistd.h> ok */
#define _lib_posix_spawn	1	/* posix_spawn exists, it works and it's worth using */
#define _socket_peek	1	/* recv(MSG_PEEK) works on socketpair() */
#define _lib_utime_now	1	/* utime works with 0 time vector */
#define _UNIV_DEFAULT	"ucb"	/* default universe name */
#define _std_cleanup	1	/* stuck with standard _cleanup */
#define _std_strcoll	1	/* standard strcoll works */
#if !_AST_no_spawnveg
#define _use_spawnveg	1
#endif

#endif
