/* : : generated from /pobj/ksh-1.0.10/ksh-1.0.10/src/lib/libast/features/wait by iffe version 2024-03-20 : : */
#ifndef _def_wait_ast
#define _def_wait_ast	1
#define _sys_types	1	/* #include <sys/types.h> ok */
#define _lib_wait	1	/* wait() in default lib(s) */
#define _lib_wait3	1	/* wait3() in default lib(s) */
#define _lib_wait4	1	/* wait4() in default lib(s) */
#define _lib_waitpid	1	/* waitpid() in default lib(s) */
#define _sys_wait	1	/* #include <sys/wait.h> ok */
#define _ok_wif	1	/* POSIX wait(2) macros ok */
#endif
